#include "chartable.h"
#include "l2f50126.h"
#include "lpc23xx.h"
#include "ssp.h"

unsigned short textcolor=0xFFFF;    // color of text
unsigned short backcolor=0x0000;    // color of background
static const unsigned char disctl[9] = {0x4C, 0x01, 0x53, 0x00, 0x02, 0xB4, 0xB0, 0x02, 0x00};
static const unsigned char gcp64_0[29] =
                      {0x11,0x27,0x3C,0x4C,0x5D,0x6C,0x78,0x84,0x90,0x99,0xA2,0xAA,0xB2,0xBA,
                       0xC0,0xC7,0xCC,0xD2,0xD7,0xDC,0xE0,0xE4,0xE8,0xED,0xF0,0xF4,0xF7,0xFB,
                       0xFE};
static const unsigned char gcp64_1[34] =
                     {0x01,0x03,0x06,0x09,0x0B,0x0E,0x10,0x13,0x15,0x17,0x19,0x1C,0x1E,0x20,
                      0x22,0x24,0x26,0x28,0x2A,0x2C,0x2D,0x2F,0x31,0x33,0x35,0x37,0x39,0x3B,
                      0x3D,0x3F,0x42,0x44,0x47,0x5E};
static const unsigned char gcp16[15] = {0x13,0x23,0x2D,0x33,0x38,0x3C,0x40,0x43,0x46,0x48,0x4A,0x4C,0x4E,0x50,0x64};

void delay_us(unsigned int us){
	unsigned long i;
	for (i=0;i<us;i++){
		//asm("nop");
	}
}

void lcd_cmd(unsigned char dat)
{
  lcd_rs_low();

  lcd_write((dat<<8)&0xff00);

  lcd_rs_high();
}

void lcd_dat0(unsigned char dat)
{
  lcd_write((dat<<8)&0xff00); 
}

void lcd_dat16(unsigned short dat)
{
  lcd_write(dat);
}

void lcd_cspulse(void)
{
 	lcd_cs_high();
  	delay_us(10);
  	lcd_cs_low();
}

void lcd_init(void)
{
	unsigned int i;
	FIO2DIR |= (1<<1); //RESET
	FIO1DIR |= (1<<7); //RS
	FIO4DIR |= (1<<21); //CS
 /*
  Init the Siemens S65 Display with EPSON controller

  (c) Christian Kranz, 10/2005 edited by Steffen L. Graf, 6/2007
*/
   lcd_reset_low();
   lcd_cs_high();
   lcd_rs_high();
   delay_us(10000);
   lcd_reset_high();
   delay_us(35000);
   lcd_cs_low();

   lcd_cmd(DATCTL);
   lcd_dat0(0x2A);  // 0x2A=565 mode, 0x0A=666mode, 0x3A=444mode

   lcd_cspulse();
  
  lcd_cmd(DISCTL);
   for (i=0; i<9; i++)
   {
   		lcd_dat0(disctl[i]);
   }
  
  lcd_cmd(GCP64);
  for (i=0; i<29; i++)
  {
    lcd_dat0(gcp64_0[i]);
    lcd_dat0(0x00);
  }
  for (i=0; i<34; i++)
  {
    lcd_dat0(gcp64_1[i]);
    lcd_dat0(0x01);
  }

  lcd_cmd(GCP16);
  for (i=0; i<15; i++)
  {
    lcd_dat0(gcp16[i]);
  }

  lcd_cmd(GSSET);
  lcd_dat0(0x00);

  lcd_cmd(OSSEL);
  lcd_dat0(0x00);

  lcd_cmd(SLPOUT);
  
//  _delay_ms(7);

  lcd_cmd(SD_CSET);
  lcd_dat0(0x08);
  lcd_dat0(0x01);
  lcd_dat0(0x8B);
  lcd_dat0(0x01);

  lcd_cmd(SD_PSET);
  lcd_dat0(0x00);
  lcd_dat0(0x8F);


  lcd_cmd(ASCSET);
  lcd_dat0(0x00);
  lcd_dat0(0xAF);
  lcd_dat0(0xAF);
  lcd_dat0(0x03);
  
  lcd_cmd(SCSTART);
  lcd_dat0(0x00);

  lcd_rs_low();
  lcd_dat0(DISON);
  
  lcd_cs_high();  // deselect display
}

void fill_screen(unsigned short color)
{
  unsigned long i;
  backcolor=color;
  lcd_cs_low();  // select display
  
  lcd_cmd(SD_CSET);
  lcd_dat0(0x08);  // start is 8, not 0
  lcd_dat0(0x01);
  lcd_dat0(0x8B);  // is 00x8B = 0x83+8
  lcd_dat0(0x01);

  lcd_cmd(SD_PSET);
  lcd_dat0(0x00);
  lcd_dat0(0xAF);
  
  lcd_cmd(RAMWR);

  // start data transmission  
  for (i=0; i<DISP_W*DISP_H; i++)
    lcd_dat16(color);
    
  lcd_cs_high();  // deselect display    
}


void put_char(unsigned char x, unsigned char y, unsigned char c) 
{
  unsigned char he,byte;
  int bit;
  x=x*CHAR_W;
  y=y*CHAR_H;
  lcd_cs_low();  // select display
  
  lcd_cmd(SD_CSET);
  lcd_dat0(0x08+x);  // start is 8, not 0
  lcd_dat0(0x01);
  lcd_dat0(0x08+x+CHAR_W-1);  
  lcd_dat0(0x01);
  
  lcd_cmd(SD_PSET);
  lcd_dat0(y);
  if (y < DISP_H-CHAR_H)
  {
    he=CHAR_H;
    lcd_dat0(y+CHAR_H-1);
  }
  else
  {
    he=DISP_H-y;
    lcd_dat0(DISP_H-1);
  }

  lcd_cmd(RAMWR);
  
  /*for (h=0; h<he; h++) // every column of the character
  {        
        ch=ascii_tab[ c-32 ][h];

        mask=0x80;
        for (p=0; p<CHAR_W; p++)  // write the pixels
        {
          if (ch&mask)
          {
            lcd_dat16(textcolor);                
          }
          else
          {
            lcd_dat16(backcolor);
          }
          mask=mask/2;
        }  // for p
  }	 */
  for (byte = 0; byte < 3; byte++) {
		for (bit = 7; bit >= 0; bit--) {
			if (ascii_tab[(int) c][byte] & (1 << bit))
				lcd_dat16(textcolor);
			else
				lcd_dat16(backcolor);

			if (bit == 4) {
				/* 4 bits are finished - start next line */
				y++;
			} else
				x++;
		}
	}

  lcd_cs_high();  // deselect display    
}

void pset(char x, char y, unsigned short col)
{
  lcd_cs_low();  // select display
  
  lcd_cmd(SD_CSET);
  lcd_dat0(0x08+x);  // start is 8, not 0
  lcd_dat0(0x01);
  lcd_dat0(0x08+x);  // end is 00x8B = 0x83+8
  lcd_dat0(0x01);
  
  lcd_cmd(SD_PSET);
  lcd_dat0(y);
  lcd_dat0(y);
  lcd_cmd(RAMWR);
  lcd_dat16(col);  
  lcd_cs_high();  // deselect display    
}

void drawline(char x1, char y1, char x2, char y2, unsigned short color) {
	char length, xTmp, yTmp, i, y, yAlt;
	short m;
	
	if(x1 == x2) {									// vertical line
		// x1|y1 must be the upper point
		if(y1 > y2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		length = y2-y1;
		for(i=0; i<=length; i++) {
			pset(x1, y1+i, color);
		}
	} else if(y1 == y2) {							// horizontal line
		// x1|y1 must be the left point
		if(x1 > x2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		length = x2-x1;
		for(i=0; i<=length; i++) {
			pset(x1+i, y1, color);
		}
	} else {
		// x1 must be smaller than x2
		if(x1 > x2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		if((y2-y1) >= (x2-x1) || (y1-y2) >= (x2-x1)) {	// angle larger or equal 45
			length = x2-x1;								// not really the length :)
			m = ((y2-y1)*200)/length;
			yAlt = y1;
			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+y1;
				
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				
				drawline(x1+i, yAlt, x1+i, y, color);
				if(length<=(y2-y1) && y1 < y2)
					yAlt = y+1;
				else if(length<=(y1-y2) && y1 > y2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		} else {										// angle smaller 45
			// y1 must be smaller than y2
			if(y1 > y2) {
				xTmp = x1;
				yTmp = y1;
				x1 = x2;
				y1 = y2;
				x2 = xTmp;
				y2 = yTmp;
			}
			length = y2-y1;
			m = ((x2-x1)*200)/length;
			yAlt = x1;
			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+x1;
				
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				
				drawline(yAlt, y1+i, y, y1+i,color);
				if(length<=(x2-x1) && x1 < x2)
					yAlt = y+1;
				else if(length<=(x1-x2) && x1 > x2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		}
	}
}   /*

void plot (unsigned char *buffer, unsigned long count, unsigned short color)
{
unsigned long cnt;
	for (cnt=1;cnt<count;cnt++){
		drawline( buffer[cnt+1], ((cnt+1)*175/count), buffer[cnt],((cnt*175)/count), color);
	}
}	 */

void displayint2(short data, char x, char y)
  {
    unsigned char byt;
	byt = data / 1000;
	data = data - byt * 1000;
	byt = data / 100;
	data = data - byt * 100;
	byt = data / 10;
	data = data - byt * 10;
	put_char(x, y, byt + 48);
	put_char(x+1, y, data + 48); 
}

void displayint4(short data, char x, char y)
  {
    unsigned char byt;
	byt = data / 1000;
	data = data - byt * 1000;
	put_char(x,y,byt + 48);
	byt = data / 100;
	data = data - byt * 100;
	put_char(x+1,y,byt + 48);
	byt = data / 10;
	data = data - byt * 10;
	put_char(x+2,y,byt + 48);
	put_char(x+3,y,data + 48); 
}

void displayint8(long data, char x, char y)
  {
    unsigned char byt;
    byt = data / 10000000;
	data = data - byt * 10000000;
	put_char(x,y,byt + 48);

    byt = data / 1000000;
	data = data - byt * 1000000;
	put_char(x+1,y,byt + 48);

    byt = data / 100000;
	data = data - byt * 100000;
	put_char(x+2,y,byt + 48);

    byt = data / 10000;
	data = data - byt * 10000;
	put_char(x+3,y,byt + 48);

	byt = data / 1000;
	data = data - byt * 1000;
	put_char(x+4,y,byt + 48);

	byt = data / 100;
	data = data - byt * 100;
	put_char(x+5,y,byt + 48);

	byt = data / 10;
	data = data - byt * 10;
	put_char(x+6,y,byt + 48);
	put_char(x+7,y,data + 48); 
}

void displaystr(char *text, char x, char y)
{
	int i=0;
	while(text[i]!=0)
	{
		put_char(x+i,y,text[i]);
		i++;
	}
}

void lcd_dump(unsigned char *text, unsigned int num)
{
	unsigned static int y;
	textcolor = RED;
	int i=0,x=0;
	while(text[i]!=0)
	{
		put_char(x+i,y,text[i]);
		i++;
	}
	if (num>0)
		displayint4(num, (x+i+1),y);
	y++;
	if (y>=28)
		y=0;
	textcolor = WHITE;
}

void fill_screen_data(unsigned short buffer[130][130])	
{
  //unsigned long i;
  int x,y;
  lcd_cs_low();  // select display
  
  lcd_cmd(SD_CSET);
  lcd_dat0(0x08);  // start is 8, not 0
  lcd_dat0(0x01);
  lcd_dat0(0x08+129); 
  lcd_dat0(0x01);

  lcd_cmd(SD_PSET);
  lcd_dat0(0x00);
  lcd_dat0(129);
  
  lcd_cmd(RAMWR);

  // start data transmission  
	for(x = 0; x < 130; x++){
		for(y = 0; y < 130; y++)
    	{
        	lcd_dat16(buffer[x][y]);
    	}
   	} 
    
  lcd_cs_high();  // deselect display    
}


