#include "lpc23xx.h"
#include <math.h>
#include "uart.h"
#include "ssp.h"
#include "l2f50126.h"

extern int SDRAM_BASE_ADDR;
extern int SRAM_BASE_ADDR;

void UNDEF_Routine(){
sendstr_uart0("undef");
}
void SWI_Routine(){
sendstr_uart0("swi");
}
void IRQ_Routine(){
sendstr_uart0("irq");
}
void FIQ_Routine(){
sendstr_uart0("fiq");
}

void init_ram_low(void){
	int i;
	PCONP|=(1<<11);	
	PINSEL5 = 0x55010115;
	PINMODE5 = 0xAAaaaaaA;
	PINSEL6 = 0x55555555;
	PINMODE6 = 0xAAAAAAAA;
	PINSEL7 = 0x55555555;
	PINMODE7 = 0xAAAAAAAA;
	PINSEL8 = 0x05555555;
	PINMODE8 = 0x0AAAAAAA;
	PINSEL9 = (1 << 18);
	PINMODE9 = 0x80000;	

	EMC_CTRL=1;			// enable EMC
	EMC_DYN_RD_CFG=1;	//Configures the dynamic memory read strategy(Command delayed strategy)
	EMC_DYN_RASCAS0|=0x303;
	EMC_DYN_RP= 4;		//RP
	EMC_DYN_RAS = 8;	//RAS
	EMC_DYN_SREX = 5;	// SREX, XSR (not found in datasheet)
	EMC_DYN_APR = 2;	//APR (not found in datasheet)
	EMC_DYN_DAL = 5;	//DAL, APW (not found in datasheet)
	EMC_DYN_WR = 3;		//WR, DPL, RWL, RDL
	EMC_DYN_RC = 11;	//
	EMC_DYN_RFC = 11;	//RFC, RC
	EMC_DYN_XSR = 5;	//XSR (not found in datasheet)
	EMC_DYN_RRD = 3;	//RRD
	EMC_DYN_MRD = 8;	//RAS
	EMC_DYN_CFG0 = (1<<14)|(3<<9)|(1<<8)|(1<<12);//8,9,14
	//64 MB (2Mx32), 4 banks, row length = 11, column length = 8, low power mode
	
	for(i = 0; i < 100; i++);
	EMC_DYN_CTRL = 0x0183; // NOP
	for(i = 0; i < 100; i++);
	EMC_DYN_CTRL=0x101; 
	EMC_DYN_RFSH = 1;
	for(i = 0; i < 100; i++);
	EMC_DYN_RFSH = 19;
	EMC_DYN_CTRL=0x81; 
	i = (*(volatile int*)(0xA0000000 | (0x32<<12)));
	EMC_DYN_CTRL = 0;
	EMC_DYN_CFG0|=0x80000;	
}

void init_ram_high(void){
	int i;
	PCONP|=(1<<11);
	PINSEL5 = 0x55010115;
	PINMODE5 = 0xAAaaaaaA;
	PINSEL6 = 0x55555555;
	PINMODE6 = 0xAAAAAAAA;
	PINSEL7 = 0x55555555;
	PINMODE7 = 0xAAAAAAAA;
	PINSEL8 = 0x05555555;
	PINMODE8 = 0x0AAAAAAA;
	PINSEL9 = (1 << 18);
	PINMODE9 = 0x80000;

	EMC_CTRL=1;			// enable EMC
	EMC_DYN_RD_CFG=1;	//Configures the dynamic memory read strategy(Command delayed strategy)
	EMC_DYN_RASCAS0|=0x303;
	EMC_DYN_RP= 4;		//RP
	EMC_DYN_RAS = 8;	//RAS
	EMC_DYN_SREX = 5;	// SREX, XSR (not found in datasheet)
	EMC_DYN_APR = 2;	//APR (not found in datasheet)
	EMC_DYN_DAL = 5;	//DAL, APW (not found in datasheet)
	EMC_DYN_WR = 3;		//WR, DPL, RWL, RDL
	EMC_DYN_RC = 11;	//
	EMC_DYN_RFC = 11;	//RFC, RC
	EMC_DYN_XSR = 5;	//XSR (not found in datasheet)
	EMC_DYN_RRD = 3;	//RRD
	EMC_DYN_MRD = 8;	//RAS
	EMC_DYN_CFG0 = (1<<14)|(3<<9)|(1<<8);//8,9,14
	//64 MB (2Mx32), 4 banks, row length = 11, column length = 8, high performance mode
	
	for(i = 0; i < 100; i++);
	EMC_DYN_CTRL = 0x0183; // NOP
	for(i = 0; i < 100; i++);
	EMC_DYN_CTRL=0x101; 
	EMC_DYN_RFSH = 1;
	for(i = 0; i < 100; i++);
	EMC_DYN_RFSH = 19;
	EMC_DYN_CTRL=0x81; 
	i = (*(volatile int*)(0xA0000000 | (0x32<<12)));
	EMC_DYN_CTRL = 0;
	EMC_DYN_CFG0|=0x80000;	
}

int test_ram(void){
	unsigned int i;
	unsigned int temp;
	for (i = 100; i < 0x100000; i+=sizeof(unsigned short))
	{
		*(unsigned short*)((unsigned int)&SDRAM_BASE_ADDR+i) = 0xaaaa; //Fill RAM with 0xaaaa
	}

	for (i = 100; i < 0x100000; i+=sizeof(unsigned short))
	{
		temp = *(unsigned short*)((unsigned int)&SDRAM_BASE_ADDR+i);
		if (temp != 0xaaaa) //if bit mask stored in RAM is wrong, send address via UART0
		{
			sendint8_uart0(i);
			send_uart0(' ');
			sendint8_uart0(temp);
			send_uart0(10);
			send_uart0(13);
		}
	}
	return(1);
}

void init_pll(void){
    SCS|=(1<<5);			// Enable main OSC   
    while(!(SCS&0x40));	// Wait until main OSC is usable   
    CLKSRCSEL = 0x1;   
	
	PLLCON = 0;
	PLLFEED = 0xAA;
	PLLFEED = 0x55;
	// 4. Disable PLL
	PLLCON = 0;
	PLLFEED = 0xAA;
	PLLFEED = 0x55;
	// 5. Select source clock for PLL
	// 6. Set PLL settings 280.1664 MHz divider = 1 multiplier = 19
	PLLCFG = (1<<16)|18;
	PLLFEED = 0xAA;
	PLLFEED = 0x55;
	// 7. Enable PLL
	PLLCON = 1;
	PLLFEED = 0xAA;
	PLLFEED = 0x55;
	// 8. Wait for the PLL to achieve lock
	while(!(PLLSTAT&(1<<26)));
	// 9. Set clk divider settings
	CCLKCFG   = 4-1;		// 1/4 Fpll - 70.0416 MHz
	//USBCLKCFG = 6-1;		// 1/6 Fpll - 48 MHz
	// 10. Connect the PLL
	PLLCON = 3;
	PLLFEED = 0xAA;
	PLLFEED = 0x55;
	
	PCLKSEL0 = 0x55555555;	//peripherals clock = 70.0416 MHz
	PCLKSEL1 = 0x55555555;
	
	MAMCR = 0x00;		// Turn MAM off (default)
	MAMTIM = 0x04;		// Set flash timing to 4 clock cycles
	MAMCR = 0x02;		// Fully enable the Memory Accleration Module
}

unsigned short colorRGB (unsigned char r, unsigned char g, unsigned char b) 
{
	unsigned short color=0;
	r /= 8;
	g /= 4;
	b /= 8;
	color = (r<<11);
	color |= (g<<5);
	color |= b;
	return color;
}

#define x_width 130
#define y_width	130
unsigned short palette[256];
unsigned short buffer[x_width][y_width];
unsigned short plasma[x_width][y_width];
/* These variables are placed iin .bss area; .bss is placed in SDRAM */

void plasma1(void)
{
	//unsigned short plasma[x_width][y_width];
	//unsigned short buffer[x_width][y_width];
	//unsigned short palette[256];
	unsigned short color;
	unsigned char x;
	unsigned char y;
	unsigned short paletteShift;
	int i;
	char temp=2;
	//Init Timer 1
	T1PR = 70000; 
	T1TCR = 1;
	T1TC = 0; 

	for (i = 0; i < 256; i++) {
		palette[i] = colorRGB(128 + 128 * sin(M_PI * i / ((float) 256 / 8.0)), 
		128 + 128 * sin(M_PI * i / ((float) 256 / 4.0)), 
		128 + 128 * sin(M_PI * i / ((float) 256 / 2.0)));
	}
	
	for(x = 0; x < x_width; x++){
   	 	for(y = 0; y < y_width; y++)
    	{
        	//the plasma buffer is a sum of sines
        	color = (
              128.0 + (128.0 * sin(x / 4.0))
            + 128.0 + (128.0 * sin(y / 2.0))
            + 128.0 + (128.0 * sin((x + y) / 4.0))
            + 128.0 + (128.0 * sin(sqrt((x * x + y * y)) / 2.0))
        	) / 4;
        	plasma[x][y] = color;        
    	} 
	}
	while(1)
	{
        //the parameter to shift the palette varies with time
		if (temp==2){
			FIO0SET = (1<<20);
			temp=1;
		}else{
			FIO0CLR = (1<<20);
			temp=2;
		}
		paletteShift = (T1TC / 16);
        
		//draw every pixel again, with the shifted palette color
		for(x = 0; x < x_width; x++){
			for(y = 0; y < y_width; y++)
			{
			buffer[x][y]=palette[(plasma[x][y] + paletteShift) % 256];       
			}
		}
		//make everything visible
		/*for(x = 0; x < x_width; x++){
			for(y = 0; y <y_width; y++)
			{
				pset(x, y, buffer[x][y]);
			}
		}*/
		fill_screen_data(buffer); //copies framebuffer to LCD
    } 
}


int main(void){
	init_ram_high();
	init_ram_low();
	init_pll();
	init_uart0(115200);
	SCS |= 1; 			//Enable FGPIO for port 0 and 1 
	init_ssp1();
	lcd_init();
	fill_screen(BLACK);	
	FIO0DIR = (1<<20); //Set P0.20 as output, for meassuring fps
	plasma1();
	while(1);
}
